@@ -6,9 +6,12 @@ import sys
 import traceback
 import typing
 
+from abc import ABC, abstractmethod
 from fastavro import schemaless_reader, schemaless_writer
+from jsonschema import validate
 
-from schema_registry.client import SchemaRegistryClient, schema
+from schema_registry.client import SchemaRegistryClient, schema, utils
+from schema_registry.client.schema import BaseSchema
 from schema_registry.client.errors import ClientError
 
 from .errors import SerializerError
@@ -30,7 +33,7 @@ class ContextStringIO(io.BytesIO):
         self.close()
 
 
-class MessageSerializer:
+class MessageSerializer(ABC):
     """
     A helper class that can serialize and deserialize messages
     Args:
@@ -49,15 +52,20 @@ class MessageSerializer:
         self.reader_schema = reader_schema
         self.return_record_name = return_record_name
 
-    def _get_encoder_func(self, avro_schema: typing.Union[schema.AvroSchema, str]) -> typing.Callable:
-        if isinstance(avro_schema, str):
-            avro_schema = schema.AvroSchema(json.loads(avro_schema))
+    @property
+    @abstractmethod
+    def _serializer_schema_type(self) -> str:
+        ...
 
-        return lambda record, fp: schemaless_writer(fp, avro_schema.schema, record)  # type: ignore
+    @abstractmethod
+    def _get_encoder_func(self, schema: BaseSchema) -> typing.Callable:
+        ...
 
-    def encode_record_with_schema(
-        self, subject: str, avro_schema: typing.Union[schema.AvroSchema, str], record: dict
-    ) -> bytes:
+    @abstractmethod
+    def _get_decoder_func(self, payload: ContextStringIO, writer_schema: BaseSchema) -> typing.Callable:
+        ...
+
+    def encode_record_with_schema(self, subject: str, schema: BaseSchema, record: dict) -> bytes:
         """
         Given a parsed avro schema, encode a record for the given subject.
         The record is expected to be a dictionary.
@@ -70,7 +78,7 @@ class MessageSerializer:
             bytes: Encoded record with schema ID as bytes
         """
         # Try to register the schema
-        schema_id = self.schemaregistry_client.register(subject, avro_schema)
+        schema_id = self.schemaregistry_client.register(subject, schema, schema_type=self._serializer_schema_type)
 
         if not schema_id:
             message = f"Unable to retrieve schema id for subject {subject}"
@@ -78,7 +86,7 @@ class MessageSerializer:
 
         # cache writer
         if not self.id_to_writers.get(schema_id):
-            self.id_to_writers[schema_id] = self._get_encoder_func(avro_schema)
+            self.id_to_writers[schema_id] = self._get_encoder_func(schema)
 
         return self.encode_record_with_schema_id(schema_id, record)
 
@@ -95,10 +103,10 @@ class MessageSerializer:
         # use slow avro
         if schema_id not in self.id_to_writers:
             try:
-                avro_schema = self.schemaregistry_client.get_by_id(schema_id)
-                if not avro_schema:
+                schema = self.schemaregistry_client.get_by_id(schema_id)
+                if not schema:
                     raise SerializerError("Schema does not exist")
-                self.id_to_writers[schema_id] = self._get_encoder_func(avro_schema)
+                self.id_to_writers[schema_id] = self._get_encoder_func(schema)
             except ClientError:
                 exc_type, exc_value, exc_traceback = sys.exc_info()
                 raise SerializerError(repr(traceback.format_exception(exc_type, exc_value, exc_traceback)))
@@ -113,24 +121,6 @@ class MessageSerializer:
 
             return outf.getvalue()
 
-    def _get_decoder_func(self, schema_id: int, payload: ContextStringIO) -> typing.Callable:
-        if schema_id in self.id_to_decoder_func:
-            return self.id_to_decoder_func[schema_id]
-
-        try:
-            writer_schema = self.schemaregistry_client.get_by_id(schema_id)
-        except ClientError as e:
-            raise SerializerError(f"unable to fetch schema with id {schema_id}: {e}")
-
-        if writer_schema is None:
-            raise SerializerError(f"unable to fetch schema with id {schema_id}")
-
-        self.id_to_decoder_func[schema_id] = lambda payload: schemaless_reader(
-            payload, writer_schema.schema, self.reader_schema, self.return_record_name
-        )
-
-        return self.id_to_decoder_func[schema_id]
-
     def decode_message(self, message: typing.Optional[bytes]) -> typing.Optional[dict]:
         """
         Decode a message from kafka that has been encoded for use with
@@ -151,6 +141,54 @@ class MessageSerializer:
             magic, schema_id = struct.unpack(">bI", payload.read(5))
             if magic != MAGIC_BYTE:
                 raise SerializerError("message does not start with magic byte")
-            decoder_func = self._get_decoder_func(schema_id, payload)
+
+            if schema_id in self.id_to_decoder_func:
+                return self.id_to_decoder_func[schema_id](payload)
+
+            try:
+                writer_schema = self.schemaregistry_client.get_by_id(schema_id)
+            except ClientError as e:
+                raise SerializerError(f"unable to fetch schema with id {schema_id}: {e}")
+
+            if writer_schema is None:
+                raise SerializerError(f"unable to fetch schema with id {schema_id}")
+
+            decoder_func = self._get_decoder_func(payload, writer_schema)
+            self.id_to_decoder_func[schema_id] = decoder_func
 
             return decoder_func(payload)
+
+
+class AvroMessageSerializer(MessageSerializer):
+    @property
+    def _serializer_schema_type(self) -> str:
+        return utils.AVRO_SCHEMA_TYPE
+
+    def _get_encoder_func(self, schema: typing.Union[BaseSchema]) -> typing.Callable:
+        return lambda record, fp: schemaless_writer(fp, schema.schema, record)  # type: ignore
+
+    def _get_decoder_func(self, payload: ContextStringIO, writer_schema: BaseSchema) -> typing.Callable:
+        return lambda payload: schemaless_reader(
+            payload, writer_schema.schema, self.reader_schema, self.return_record_name
+        )  # type: ignore
+
+
+class JsonMessageSerializer(MessageSerializer):
+    @property
+    def _serializer_schema_type(self) -> str:
+        return utils.JSON_SCHEMA_TYPE
+
+    def _get_encoder_func(self, schema: typing.Union[BaseSchema]) -> typing.Callable:
+        def json_encoder_func(record: dict, fp: ContextStringIO) -> typing.Any:
+            validate(record, schema.schema)
+            fp.write(json.dumps(record).encode())
+
+        return json_encoder_func
+
+    def _get_decoder_func(self, payload: ContextStringIO, writer_schema: BaseSchema) -> typing.Callable:
+        def json_decoder_func(payload: typing.Union[str, bytes]) -> typing.Any:
+            obj = json.load(payload)  # type: ignore
+            validate(obj, writer_schema.schema)  # type: ignore
+            return obj
+
+        return json_decoder_func
